unit SingletonClassUnit;

// code from http://www.skybound.nl/articles/delphi-singleton/
// based on  http://community.borland.com/article/0,1410,22576,00.html
// Modified by Ruud Vlaming

interface

uses SysUtils;

type
  ESingletonException = class(Exception);

  TSingleton = class(TObject)
  private
    class function RefCount: Cardinal;
  public
    constructor Create;
    class function NewInstance: TObject; override;
    procedure FreeInstance; override;
    procedure CreateSingleton; virtual; abstract;
  end;

implementation

uses Classes;

type
  // Define structure that contains the reference count and object pointer.
  PssSingletonInstance = ^TssSingletonInstance;
  TssSingletonInstance =
  record
    refcount: Cardinal;
    instance: TObject;
  end;

// The instances: classname => TssSingletonInstance pairs
var instances: TStrings;

procedure CleanupInstances;
var
  i: integer;
begin
for i := instances.Count-1 downto 0 do
  begin
  if PssSingletonInstance(instances.Objects[i])^.refcount > 1 then
    begin
    PssSingletonInstance(instances.Objects[i]).instance.Free;
    if (i <= instances.Count) then
      begin
      Dispose(PssSingletonInstance(instances.Objects[i]));
      instances.Delete(i);
      end;
    end
  else PssSingletonInstance(instances.Objects[i]).instance.Free;
  end;
end;

constructor TSingleton.Create;
begin
inherited;
if (RefCount<=1) then CreateSingleton;
end;

procedure TSingleton.FreeInstance;
var i: integer;
    instance: PssSingletonInstance;
begin
i:=instances.IndexOf(ClassName);
if i<0 then
begin // no instance found!
  inherited;
  Exit;
end;
instance:=PssSingletonInstance(instances.Objects[i]);
if (instance^.refcount > 1)
then Dec(instance^.refcount)
else
  begin
  inherited FreeInstance; // free instance here
  // free and remove the PssSingletonInstance from the list.
  Dispose(PssSingletonInstance(instances.Objects[i]));
  instances.Delete(i);
  end;
end;

class function TSingleton.NewInstance: TObject;
var i: integer;
    instance: PssSingletonInstance;
begin
i:=instances.IndexOf(ClassName);
if i>=0 then
  begin // instance found
  instance:=PssSingletonInstance(instances.Objects[i]);
  end
else
  begin
  new(instance);
  instance^.instance:=inherited NewInstance;
  instance^.refcount:=0;
  instances.AddObject(ClassName,TObject(instance));
  end;
Inc(instance^.refcount);
Result:=instance^.instance;
end;

class function TSingleton.RefCount: Cardinal;
var i: integer;
    instance: PssSingletonInstance;
begin
  i:=instances.IndexOf(ClassName);
  if i>=0 then
    begin // instance found
    instance := PssSingletonInstance(instances.Objects[i]);
    Result := instance^.refcount;
    end
  else raise ESingletonException.Create('Class "' + ClassName +'" not found in instances list!');
end;

initialization
instances:=TStringList.Create;

finalization
CleanupInstances;

end.

